import pymunk
import pygame
import os
import math
import random
from loader import loader
from Config import *
from pymunk import Vec2d
from constants import *
from State import State

class Block:
	def __init__(self,space,index,x,y=500):
		self.space = space
		self.state = State()
		self.radius = None
		self.anchor = None
		self.line_to = None
		self.children = []
		if index == 0: # box
			points = map(Vec2d, [(-60,-20),(-60,25),(60,25),(60,-20)])
			self.shape = self.create_poly(points,5.0,(x,y))
			self.size = [120,50]
		elif index == 1: # ramp
			points = map(Vec2d, [(-60,25),(60,25),(60,-20)])
			self.shape = self.create_poly(points,5.0,(x,y))
			self.size = [120,50]
		elif index == 2: # ball
			self.radius = 22
			self.shape = self.create_ball((x,y),0.01,self.radius)
			self.size = [60,60]
		elif index == -1: # spacer
			points = map(Vec2d, [(-60,-20),(-60,25),(60,25),(60,-20)])
			self.shape = self.create_poly(points,5.0,(x,y),SPACER)
			self.size = [120,50]
		elif index == 3:# ramp down
			points = map(Vec2d, [(-60,-20),(-60,25),(60,25)])
			self.shape = self.create_poly(points,5.0,(x,y))
			self.size = [120,50]
		elif index == 4:# power ramp down
			points = map(Vec2d, [(-60,-20),(-60,25),(60,25)])
			self.shape = self.create_poly(points,5.0,(x,y),RIGHTACCEL)
			self.size = [120,50]
		elif index == 5:# power ramp up
			points = map(Vec2d, [(-60,25),(60,25),(60,-20)])
			self.shape = self.create_poly(points,5.0,(x,y),RIGHTACCEL)
			self.size = [120,50]
		elif index == 6: # wideslow
			points = map(Vec2d, [(-87,-10),(-87,15),(87,15),(87,-10)])
			self.shape = self.create_poly(points,5.0,(x,y))
			self.shape.body.friction = 100
			self.size = [174,30]
		elif index == 7: # square
			points = map(Vec2d, [(-25,-20),(-25,25),(25,25),(25,-20)])
			self.shape = self.create_poly(points,0.001,(x,y))
			self.size = [50,50]
		elif index == 8: # bouncer
			self.radius = 35
			self.shape = self.create_ball((x,y),0.001,self.radius,BOUNCE)
			self.size = [70,70]
			# anchor:
			anchor_body = pymunk.Body(100,100)
			self.chain_length = 20
			anchor_body.position = Vec2d((x,y))
			joint = pymunk.SlideJoint(self.shape.body,anchor_body,(0,0),(0,0),0,self.chain_length)
			self.space.add(joint)
			self.children.append(anchor_body)
			self.children.append(joint)
			self.line_to = (x,y)
		elif index == 9: # downball
			self.radius = 22
			self.shape = self.create_ball((x,y),0.0001,self.radius,DOWN)
			self.size = [60,60]
			# anchor:
			anchor_body = pymunk.Body(100,100)
			self.chain_length = 5
			anchor_body.position = Vec2d((x,y))
			joint = pymunk.SlideJoint(self.shape.body,anchor_body,(0,0),(0,0),0,self.chain_length)
			self.space.add(joint)
			self.children.append(anchor_body)
			self.children.append(joint)
			self.line_to = (x,y)
		elif index == 10: # widestop
			points = map(Vec2d, [(-190,-10),(-190,15),(190,15),(190,-10)])
			self.shape = self.create_poly(points,5.0,(x,y),SLOW)
			self.size = [390,30]
		elif index == 11: # upball
			self.radius = 15
			self.shape = self.create_ball((x,y),0.0001,self.radius,UP)
			self.size = [40,40]
			# anchor:
			anchor_body = pymunk.Body(100,100)
			self.chain_length = 10
			anchor_body.position = Vec2d((x,y))
			joint = pymunk.SlideJoint(self.shape.body,anchor_body,(0,0),(0,0),0,self.chain_length)
			self.space.add(joint)
			self.children.append(anchor_body)
			self.children.append(joint)
			self.line_to = (x,y)
		elif index == 12: # upball
			self.radius = 15
			self.shape = self.create_ball((x,y),0.0001,self.radius,LEFTACCEL)
			self.size = [40,40]
			# anchor:
			anchor_body = pymunk.Body(100,100)
			self.chain_length = 10
			anchor_body.position = Vec2d((x,y))
			joint = pymunk.SlideJoint(self.shape.body,anchor_body,(0,0),(0,0),0,self.chain_length)
			self.space.add(joint)
			self.children.append(anchor_body)
			self.children.append(joint)
			self.line_to = (x,y)
			
		self.old_angle = self.shape.body.angle
		self.index=index
		if index != -1:
			load = loader()
			surfs = load.load(os.path.join("res","art","tiles"))
			self.orig = surfs[index]
			self.surf = pygame.transform.rotate(self.orig,0)
			self.surf_size = self.orig.get_size()
	
	def update(self):
		if self.old_angle != self.shape.body.angle and not self.radius:
			self.old_angle = self.shape.body.angle
			if self.old_angle > math.pi*2:
				self.old_angle -= math.pi*2
			if self.index != 2:
				self.surf = pygame.transform.rotate(self.orig,-self.old_angle * 360 / (2 * math.pi))
		if self.shape.body.position[1] > ScreenSize[1]:
			return False
		else:
			return True
	
	def draw(self,surf,x):
		if self.line_to:
			pygame.draw.line(surf,(32,32,32),self.shape.body.position,self.line_to,3)
		if self.state.quality > 0:
			if not self.shape.collision_type == SPACER and self.shape.body.position[0]-x > -self.size[0] and self.shape.body.position[0]-x-self.surf_size[0] < ScreenSize[0]:
				surf.blit(self.surf,(self.shape.body.position[0]-x-self.size[0]/2,self.shape.body.position[1]-self.size[1]/2))
		else:
			if not self.radius:
				self.temp_x = x
				pygame.draw.lines(surf,(0,0,0),True,map(self.adj_by_x, self.shape.get_points()))
			else:
				pygame.draw.circle(surf,(0,0,0),(self.shape.body.position[0]-x,self.shape.body.position[1]), self.radius, 1)

	def adj_by_x(self,pair):
		return (pair[0]-self.temp_x,pair[1])

	def create_poly(self, points, mass = 100, pos = (0,0),type=BLOCK):
		moment = pymunk.moment_for_poly(mass,points, Vec2d(0,0))   
		if mass > 0.001:
			moment += 5000 
		#moment = 1000
		body = pymunk.Body(mass, moment)
		body.position = Vec2d(pos)       
		
		shape = pymunk.Poly(body, points, Vec2d(0,0))
		shape.friction = 0.5
		shape.collision_type = type
		self.space.add(body, shape)
		return shape
	
	def create_ball(self, point, mass=1.0, radius=15.0,type=PLAYER):
		moment = pymunk.moment_for_circle(mass, radius, 0.0, Vec2d(0,0))
		ball_body = pymunk.Body(mass, moment)
		ball_body.position = Vec2d(point)
		ball_shape = pymunk.Circle(ball_body, radius, Vec2d(0,0))
		ball_shape.friction = 1.5
		ball_shape.collision_type = type
		self.space.add(ball_body, ball_shape)
		return ball_shape