import pygame
import random
import loader
import os
from Config import *
from constants import *
from State import State

import pymunk
import math
import pymunk._chipmunk as cp

class PhysicsManager:
	def __init__(self):
		pymunk.init_pymunk()
		self.space = pymunk.Space()
		self.space.gravity = (0,100)
		self.do_reset = False
		self.state = State()
		#self.space.damping = 0.99
	
	def setup_collisions(self):
		self.space.add_collisionpair_func(PLAYER,SPACER,None)
		self.space.add_collisionpair_func(PLAYER,BUCKETWALL,self.reset)
		self.space.add_collisionpair_func(BLOCK,BUCKETWALL,self.bounce_out_of_bucket)
		self.space.add_collisionpair_func(RIGHTACCEL,BUCKETWALL,self.bounce_out_of_bucket)
		self.space.add_collisionpair_func(PLAYER,LEFTACCEL,self.left_accel)
		self.space.add_collisionpair_func(PLAYER,RIGHTACCEL,self.right_accel)
		self.space.add_collisionpair_func(PLAYER,BOUNCE,self.bounce)
		self.space.add_collisionpair_func(PLAYER,DOWN,self.down)
		self.space.add_collisionpair_func(PLAYER,SLOW,self.slow)
		self.space.add_collisionpair_func(PLAYER,UP,self.up)
		self.space.add_collisionpair_func(PLAYER,BOUNCEWALL,self.reflect)
		self.space.add_collisionpair_func(PLAYER,BACKBOUNCEWALL,self.reflectback)
		
	def reset(self,a,b,c,n,d):
		if not self.state.in_victory:
			self.state.sound.play("victory")
			self.do_reset = True
		return False
		
	def left_accel(self,a,b,c,n,d):
		self.state.sound.play("ricochet1")
		a.body.apply_impulse((-1,0),(0,0))
		return True
		
	def slow(self,a,b,c,n,d):
		a.body.apply_impulse((0,0.1),(0,0))
		return True
		
	def reflect(self,a,b,c,n,d):
		a.body.apply_impulse((-0.5,0),(0,0))
		return True
		
	def reflectback(self,a,b,c,n,d):
		a.body.apply_impulse((0.5,0),(0,0))
		return True
		
	def down(self,a,b,c,n,d):
		self.state.sound.play("ricochet3")
		a.body.apply_impulse((0,1),(0,0))
		return True
		
	def up(self,a,b,c,n,d):
		self.state.sound.play("ricochet4")
		a.body.apply_impulse((0,-2),(0,0))
		return True
		
	def right_accel(self,a,b,c,n,d):
		self.state.sound.play("ricochet1")
		a.body.apply_impulse((0.5,0),(0,0))
		return True
		
	def bounce(self,a,b,c,n,d):
		x = (a.body.position[0] - c[0].position[0])/a.radius * 5
		y = (a.body.position[1] - c[0].position[1])/a.radius * 5
		if y > 0:
			y *= 2
		print (x,y)
		self.state.sound.play("ricochet2")
		a.body.apply_impulse((x,y),(0,0))
		b.body.apply_impulse((-x/2,-y/2),(0,0))
		return True
		
	def bounce_out_of_bucket(self,a,b,contacts,normal,data):
		a.body.apply_impulse((-10,-10),(0,0))
		return False
	
	def update(self):
		if self.do_reset:
			self.reset_func()
			self.do_reset = False
		for i in range(0,3):
			self.space.step(1/40.0)
		
	def draw(self,surf):
		return True